/* fdformat.c  -  Low-level formats a floppy disk. */

/* Atari version speciality:
 * 
 * With the -t option, you can specify one or two twist factors. (If
 * there's only one, this one is used for both.) These are for track
 * twisting, i.e.  the increment of the offset of sector 1 to the
 * index mark. If this offset is choosen right, as little time as
 * possible is wasted after a track change until sector 1 is under the
 * head. The first twist factor is used between tracks on different
 * sides of the disk, but on the same cylinder. The second is applied
 * between tracks on different cylinders. In the last case, the FDC
 * has to do an additional step to the new track, so the delay between
 * end and start of reading is a bit more than without this stepping.
 *
 * As usual, the -n option suppresses verifying the disk.
 *
 * (Atari adaption by Roman.Hodek@informatik.uni-erlangen.de)
 */
/*
** 2/97	Atari speciality removed :-), to make it work on amigas.
**	Reformatted the code (only to suit personal preferences).
**      Now can dump (! not create) a root and a boot block of an Amiga
**      FFS.
**      The Reason for this:
**      kernel ioctls are still (since 1.3, I think) different from
**      i386 or atari linuxes, so original fdformat doesn't work on
**      Amigas any more, I was basically bored rebooting every time
**      I needed a FFS floppy, nothing more.
**
**      rasp@zen.co.uk
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <linux/fd.h>
#include <asm/amifd.h>
#include <linux/major.h>
#include <linux/fs.h>
#include "root_boot.h"

static int ctrl;
struct floppy_struct param;

void install_disk();

#define SECTOR_SIZE 512
#define PERROR(msg) { perror(msg); exit(1); }



void 
format_disk( char *name)
{	
unsigned long track;

printf( "Formatting ... \n" );
if (ioctl( ctrl, FDFMTBEG, NULL ) < 0)
	PERROR( "\nioctl(FDFMTBEG)" );

for( track = 0; track < param.track*param.head; track++ ) 
	{
	if (ioctl( ctrl, FDFMTTRK, track) < 0)
		PERROR( "\nioctl(FDFMTTRK)" );
	printf( "Cyl.:%3d Head:%2d\r", (int)track/2,(int)track%2 );
	fflush( stdout );
	}
if (ioctl( ctrl, FDFMTEND, NULL) < 0)
	PERROR( "\nioctl(FDFMTEND)" );
printf( "\r                 \rdone\n" );
}


void 
verify_disk( char *name )
{
unsigned char *data;
int cyl_size, cyl, count;

cyl_size = param.sect*param.head*512;
if (!(data = (unsigned char *)malloc( cyl_size )))
	PERROR( "malloc" );

printf( "\rVerifying ... " );
fflush( stdout );
	
for( cyl = 0; cyl < param.track; cyl++ ) 
	{
	printf( "%3d\b\b\b", cyl );
	fflush( stdout );
	if (read( ctrl, data, cyl_size ) != cyl_size)
		PERROR( "read" );
	for( count = 0; count < cyl_size; count++ ) 
		{
		if (data[count] != FD_FILL_BYTE) 
			{
			printf( "bad data in cyl %d\nContinuing ... ", cyl );
			fflush( stdout );
			break;
			}
		}
    	}
printf( "done\n" );
}


void 
usage( char *name )
{	
char *this;

if ((this = strrchr( name, '/' )))
	name = this+1;
fprintf( stderr, "usage: %s [-n] [-a] -d device\n-a(ffs) just for 880k disks !\n",name );
exit( 1 );
}

extern char *optarg;
extern int optind;

int main( int argc, char **argv )
{
int verify;
char *name;
struct stat stbuf;
char opt;
char *device = 0;
int affs =0; 
name = argv[0];
verify = 1;

while(( opt = getopt(argc,argv,"nad:")) != EOF)
	switch(opt)
		{
		case 'n': verify = 0;
			  break;
		case 'a': affs = 1;
			  break;
		case 'd': device = strdup(optarg);
			  break;
		default : usage(name);
			  break;
		}
if(!device)
	usage(name);
if (stat( device, &stbuf ))
	PERROR( "stat" );
if (!S_ISBLK(stbuf.st_mode) || MAJOR(stbuf.st_rdev) != FLOPPY_MAJOR) 
	{
	fprintf( stderr, "%s is no floppy device!\n", device );
	exit( 1 );
	}
if ((ctrl = open( device, O_RDWR|O_EXCL )) < 0)
	PERROR( "open" );
if (ioctl( ctrl, FDGETPRM, (int)&param ) < 0)
	PERROR( "ioctl(FDGETPRM)" );

printf("%sle-sided, %d tracks, %d sec/track."
		   " Total capacity %d kB.\n",
		   param.head ? "Doub" : "Sing",
		   param.track, param.sect, 
		   param.size >> 1 );
	
printf("Press return, when ready..\n");
getchar();
format_disk( device );
if (verify)
     verify_disk( device );
if(affs)
    install_disk();
if (close( ctrl ) < 0)
	PERROR( "close" );

return( 0 );
}

void
install_disk()
{
FILE *out;

if(!(out = fdopen(ctrl,"w")))
	exit(0);

printf("installing affs\n");	
fseek(out,OFF0,0);
fwrite(ARR_0,1,OFF0SIZE,out);
fseek(out,OFF1,0);
fwrite(ARR_1,1,OFF1SIZE,out);
fflush(out);
}
